/***************************************************************************
 * qlocx_adc.c 				                                                
 *											
 *  
 *  
 *                                                                                      
 * Functional level: 1									
 *											
 * Prepared: Dirk Handzic							
 *																					
 * 											
 * Rev	Date	Author	Ext int. (Y/N) 	Reason/description
 * 0.1  201029  DH	Y        	Initial version
 *
 *  
 * Analogue input handler 										
 ***************************************************************************/


#include <net/socket.h>
#include <net/lwm2m.h>
#include <nrf9160.h>
#include <stdio.h>
#include <string.h>
#include <drivers/adc.h>
#include <drivers/sensor.h>
#include <zephyr.h>
#include <logging/log.h>
#include <hal/nrf_saadc.h>
#include  "qlocx_adc.h"

LOG_MODULE_REGISTER(app_qlocx_adc, CONFIG_APP_LOG_LEVEL);

struct device *adc_dev;

#define ADC_DEVICE_NAME DT_ADC_0_NAME
#define ADC_RESOLUTION 12
#define ADC_GAIN ADC_GAIN_1_6
#define ADC_REFERENCE ADC_REF_INTERNAL
#define ADC_ACQUISITION_TIME ADC_ACQ_TIME(ADC_ACQ_TIME_MICROSECONDS, 40)
#define ADC_1ST_CHANNEL_ID 0
#define ADC_1ST_CHANNEL_INPUT NRF_SAADC_INPUT_AIN3        // temperature
#define ADC_2ND_CHANNEL_ID 2  
#define ADC_2ND_CHANNEL_INPUT NRF_SAADC_INPUT_AIN4        // battery voltage

static const struct adc_channel_cfg m_1st_channel_cfg = {
	.gain = ADC_GAIN,
	.reference = ADC_REFERENCE,
	.acquisition_time = ADC_ACQUISITION_TIME,
	.channel_id = ADC_1ST_CHANNEL_ID,
#if defined(CONFIG_ADC_CONFIGURABLE_INPUTS)
	.input_positive = ADC_1ST_CHANNEL_INPUT,
#endif
};

static const struct adc_channel_cfg m_2nd_channel_cfg = {
	.gain = ADC_GAIN,
	.reference = ADC_REFERENCE,
	.acquisition_time = ADC_ACQUISITION_TIME,
	.channel_id = ADC_2ND_CHANNEL_ID,
#if defined(CONFIG_ADC_CONFIGURABLE_INPUTS)
	.input_positive = ADC_2ND_CHANNEL_INPUT,
#endif
};

#define BUFFER_SIZE 2
static int16_t m_sample_buffer[BUFFER_SIZE];

int adc_sample(void)
{
int ret;


	const struct adc_sequence sequence = {
		.channels = BIT(ADC_1ST_CHANNEL_ID) | BIT(ADC_2ND_CHANNEL_ID),
		.buffer = m_sample_buffer,
		.buffer_size = sizeof(m_sample_buffer),
		.resolution = ADC_RESOLUTION,
	};

	if (!adc_dev) {
		return -1;
	}

	ret = adc_read(adc_dev, &sequence);
	printk("ADC read err: %d\n", ret);

	/* Print the AIN0 values */
	for (int i = 0; i < BUFFER_SIZE; i++) {
		float adc_voltage = 0;
		adc_voltage = (float)(((float)m_sample_buffer[i] / 4095.0f) *
				      3600.0f);
                #ifdef CONSOLE_PRINT  
		printk("ADC raw value: %d\n", m_sample_buffer[i]);
		printf("Measured voltage: %f mV\n", adc_voltage);
                #endif
	}

	return ret;
}

int qlocx_init_adc(void)
{
int err;

	adc_dev = device_get_binding("ADC_0");
	LOG_INF("%s <NTC> temperature sensor %s",
		adc_dev ? "Found" : "Did not find", "ADC_0");


	if (!adc_dev) {
		LOG_ERR("No temperature device found.");
	}

        err = adc_channel_setup(adc_dev, &m_1st_channel_cfg);
	if (err) {
		LOG_ERR("Error in AN3 adc setup: %d\n", err);
	}
         err = adc_channel_setup(adc_dev, &m_2nd_channel_cfg);
	if (err) {
		LOG_ERR("Error in AN4 adc setup: %d\n", err);
	}
       
        
        /* Trigger offset calibration
	 * As this generates a _DONE and _RESULT event
	 * the first result will be incorrect.
	 */
	NRF_SAADC_NS->TASKS_CALIBRATEOFFSET = 1;
        err = adc_sample();

	return 0;
}

// ch = 0 temperature
// ch = 1 voltage
int   qlocx_get_adc(uint8_t ch)
{
int err;
int result;

    err = adc_sample();
    if(ch < 2){
        result = m_sample_buffer[ch];
    }
    else{
        result = 0xffff;
    }

    return(result);
}


// Trigger offset calibration
// As this generates a _DONE and _RESULT event
// the first result will be incorrect.
	
void  qlocx_offset_adc(void)
{
int err;

	NRF_SAADC_NS->TASKS_CALIBRATEOFFSET = 1;
        err = adc_sample();
}