/***************************************************************************
 * qlocx_door.c 				                                                
 *											
 *  
 *  
 *                                                                                      
 * Functional level: 1									
 *											
 * Prepared: Dirk Handzic							
 *																					
 * 											
 * Rev	Date	Author	Ext int. (Y/N) 	Reason/description
 * 0.2  201214  DH      N               add door position to struct
 * 0.1  201106  DH	Y        	Initial version
 *
 *  
 * Door lock handler 										
 ***************************************************************************/
#include        <zephyr.h>
#include        <stdlib.h>
#include        <nrf9160.h>
#include        <drivers/spi.h>
#include        <drivers/gpio.h>
#include        <zephyr.h>
#include        <net/lwm2m.h>
#include        <stdint.h>
#include        <init.h>
#include        <stdio.h>
#include	"qlocx_digio.h"
#include	"qlocx_door.h"
#include        <logging/log.h>
#include        "lwm2m_object.h"
#include        "lwm2m_engine.h"

//*************************** Definitions ***********************************

#define CONFIG_APP_DOOR_PROC_TIME 5         // in ms
#define MAX_DOORS                 16





static Q_DOOR door[MAX_DOORS];

LOG_MODULE_REGISTER(app_qlocx_door, CONFIG_APP_LOG_LEVEL);



static struct k_work door_data_work;


static void  qlocx_door_handler(struct k_work *work)
{
uint16_t i, active;

    i = 0;
    active = 99;
    qlocx_digio_PwrOn(true);
    do{
        if(door[i].state == DOOR_LOGIC){
            active = i;
        }
        i++;

    }while((i < MAX_DOORS) && (active == 99));
#ifdef CONSOLE_PRINT
    printk("Active door:%d\n", active); 
#endif
    if(door[active].state == DOOR_LOGIC){
        switch(door[active].logic_index){
            case 1:
                break;

            default:
                door[active].state = DOOR_OPEN_DELAY;                
                break;
        }
    }

    if(door[active].state == DOOR_OPEN_DELAY){
        k_sleep(K_MSEC(door[active].delay));
        door[active].state = DOOR_OPEN;
        #ifdef CONSOLE_PRINT
        printk("Door:%d lock active\n", active);
        #endif
        digio_Out(active, true);
    }     

    if(door[active].state == DOOR_OPEN){
        digio_Out(active, true);            
        k_sleep(K_MSEC(door[active].duration));
        door[active].state = DOOR_CLOSE_DELAY;
     }   

    if(door[active].state == DOOR_CLOSE_DELAY){
        digio_Out(active, false);    
        door[active].state = DOOR_IDLE;
        qlocx_digio_PwrOn(false);
        #ifdef CONSOLE_PRINT
        printk("Door:%d lock passive\n", active);
        #endif
    }
}


static int door_open_cb(uint16_t obj_inst_id, uint16_t res_id, uint16_t res_inst_id,
			uint8_t *data, uint16_t data_len,
			bool last_block, size_t total_size)
{

char * data_ptr;

        //first parameter
	door[obj_inst_id].delay = atoi(data);
        data_ptr = strtok(data," ,\t");
        data_ptr += (strlen(data) + 1);
        data = data_ptr; 
        //second parameter
        door[obj_inst_id].duration = atoi(data);
        data_ptr = strtok(data," ,\t");
        data_ptr += (strlen(data) + 1);
        data = data_ptr;
        // third parameter
        door[obj_inst_id].logic_index = atoi(data);

        door[obj_inst_id].state = DOOR_LOGIC;
        
        k_work_submit(&door_data_work);

	return 0;
}

static void * door_position_cb(uint16_t obj_inst_id, uint16_t res_id, uint16_t res_inst_id,
			  size_t *data_len)
{
uint8_t  buffer[20]={0};
bool  position;

        position = digio_In(obj_inst_id);
        door[obj_inst_id].is_open = position;
        sprintf(buffer,"10351/%d/50", obj_inst_id);
        lwm2m_engine_set_bool(buffer, position);
        #ifdef CONSOLE_PRINT
        printk("Door:%d position: %s\n", obj_inst_id, position ? "true" : "false");
	#endif
        return (&door[obj_inst_id].is_open);
}

int qlocx_init_door(void)
{
uint32_t i;
uint8_t  buffer[20] = {0};

	
        k_work_init(&door_data_work, qlocx_door_handler);
	// create door control objects
        for(i = 0; i < MAX_DOORS; i++){
            sprintf(buffer,"10351/%d",i);
            lwm2m_engine_create_obj_inst(buffer);
            sprintf(buffer,"10351/%d/100",i);
            lwm2m_engine_register_post_write_callback(buffer, door_open_cb);
            sprintf(buffer,"10351/%d/50",i);
            lwm2m_engine_register_read_callback(buffer, door_position_cb);
            door[i].state = DOOR_IDLE;
        }
	
	
	return 0;
}

void  qlocx_test_door(Q_DOOR * test, uint8_t ch)
{
       door[ch].logic_index = test->logic_index;
       door[ch].delay = test->delay;
       door[ch].duration = test->duration;
       door[ch].state = test->state;
       k_work_submit(&door_data_work);
}

bool  qlocx_test_position(uint8_t ch)
{
    door_position_cb(ch, 0, 0, NULL);
    return(door[ch].is_open);

}