/***************************************************************************
 * qlocx_volt.c 				                                                
 *											
 *  
 *  
 *                                                                                      
 * Functional level: 1									
 *											
 * Prepared: Dirk Handzic							
 *																					
 * 											
 * Rev	Date	Author	Ext int. (Y/N) 	Reason/description
 * 0.3  210114  DH      Y               disable PWR_INP after measuring board voltage
 * 0.2  210112  DH      Y               enable PWR_INP before measuring board voltage
 * 0.1  201111  DH	Y        	Initial version
 *
 *  
 * Board voltage input handler 										
 ***************************************************************************/


#include <net/socket.h>
#include <net/lwm2m.h>
#include <nrf9160.h>
#include <stdio.h>
#include <string.h>
#include <drivers/adc.h>
#include <drivers/sensor.h>
#include <zephyr.h>
#include <logging/log.h>
#include  "qlocx_volt.h"
#include  "qlocx_adc.h"

LOG_MODULE_REGISTER(app_qlocx_volt, CONFIG_APP_LOG_LEVEL);

static struct float32_value volt_float = { 9, 500000 };
#if ADD_TIMESTAMPS
static int32_t timestamp;
#endif

static int read_voltage(struct float32_value *float_val)
{

int volt;
struct sensor_value volt_val;


        volt = (qlocx_get_adc(1) * 360/4095) * 5.19753;

        volt_val.val1 = volt / 100;
        volt_val.val2 = (volt - (volt_val.val1 * 100)) * 10000;
#ifdef CONSOLE_PRINT
	printk("Battery: read %d.%d V\n",  volt_val.val1, volt_val.val2);
#endif
	float_val->val1 = volt_val.val1;
	float_val->val2 = volt_val.val2;

	return 0;
}



static void *volt_read_cb(uint16_t obj_inst_id, uint16_t res_id, uint16_t res_inst_id,
			  size_t *data_len)
{
	int32_t ts;
        int err;


	/* Only object instance 0 is currently used */
	if (obj_inst_id != 0) {
		*data_len = 0;
		return NULL;
	}


	/*
	 * No need to check if read was successful, just reuse the
	 * previous value which is already stored at volt_float.
	 * This is because there is currently no way to report read_cb
	 * failures to the LWM2M engine.
	 */
        qlocx_digio_InpPwrOn(true);
        k_sleep(K_MSEC(10));
        err = adc_sample();
	read_voltage( &volt_float);

	lwm2m_engine_set_float32("3316/0/5700", &volt_float);
	*data_len = sizeof(volt_float);
        qlocx_digio_InpPwrOn(false);
	#if ADD_TIMESTAMPS
        /* get current time from device */
	lwm2m_engine_get_s32("3/0/13", &ts);        
	/* set timestamp */
	lwm2m_engine_set_s32("3316/0/5518", ts);
        #endif
	return &volt_float;
}

int qlocx_init_volt(void)
{
	
       
        // Battery voltage object
        lwm2m_engine_create_obj_inst("3316/0");
	lwm2m_engine_register_read_callback("3316/0/5700", volt_read_cb);
        #if ADD_TIMESTAMPS
	lwm2m_engine_set_res_data("3316/0/5518",
				  &timestamp, sizeof(timestamp), 0);
        #endif
	return 0;
}


float32_value_t qlocx_test_volt(void)
{
float32_value_t voltage;
int err;

   err = adc_sample();
   read_voltage( &voltage);
   return(voltage);
  
}